@@ -5,6 +5,8 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
+
 import itertools
 import os
 
@@ -95,7 +97,7 @@ def clonenarrowcmd(orig, ui, repo, *args, **opts):
         ui.status(_(b"reading narrowspec from '%s'\n") % filepath)
         try:
             fdata = util.readfile(filepath)
-        except IOError as inst:
+        except OSError as inst:
             raise error.Abort(
                 _(b"cannot read narrowspecs from '%s': %s")
                 % (filepath, encoding.strtolocal(inst.strerror))
@@ -296,7 +298,7 @@ def _narrow(
                     for file_ in entry.files():
                         todelete.append(file_.unencoded_path)
             elif entry.is_manifestlog:
-                dir = entry.target_id
+                dir = entry.target_id[:-1]
                 dirs = sorted(pathutil.dirs({dir})) + [dir]
                 include = True
                 for d in dirs:
@@ -530,7 +532,7 @@ def trackedcmd(ui, repo, remotepath=None, *pats, **opts):
         filepath = os.path.join(encoding.getcwd(), newrules)
         try:
             fdata = util.readfile(filepath)
-        except IOError as inst:
+        except OSError as inst:
             raise error.StorageError(
                 _(b"cannot read narrowspecs from '%s': %s")
                 % (filepath, encoding.strtolocal(inst.strerror))
