@@ -5,6 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
 
 import itertools
 import re
@@ -12,12 +13,7 @@ import textwrap
 
 from typing import (
     Callable,
-    Dict,
     Iterable,
-    List,
-    Optional,
-    Set,
-    Tuple,
     Union,
     cast,
 )
@@ -26,7 +22,6 @@ from .i18n import (
     _,
     gettext,
 )
-from .pycompat import getattr
 from . import (
     cmdutil,
     encoding,
@@ -43,7 +38,6 @@ from . import (
     templatefuncs,
     templatekw,
     ui as uimod,
-    util,
 )
 from .hgweb import webcommands
 from .utils import (
@@ -55,13 +49,13 @@ from .utils import (
 _DocLoader = Callable[[uimod.ui], bytes]
 # Old extensions may not register with a category
 _HelpEntry = Union["_HelpEntryNoCategory", "_HelpEntryWithCategory"]
-_HelpEntryNoCategory = Tuple[List[bytes], bytes, _DocLoader]
-_HelpEntryWithCategory = Tuple[List[bytes], bytes, _DocLoader, bytes]
+_HelpEntryNoCategory = tuple[list[bytes], bytes, _DocLoader]
+_HelpEntryWithCategory = tuple[list[bytes], bytes, _DocLoader, bytes]
 _SelectFn = Callable[[object], bool]
-_SynonymTable = Dict[bytes, List[bytes]]
+_SynonymTable = dict[bytes, list[bytes]]
 _TopicHook = Callable[[uimod.ui, bytes, bytes], bytes]
 
-_exclkeywords: Set[bytes] = {
+_exclkeywords: set[bytes] = {
     b"(ADVANCED)",
     b"(DEPRECATED)",
     b"(EXPERIMENTAL)",
@@ -77,7 +71,7 @@ _exclkeywords: Set[bytes] = {
 # Extensions with custom categories should insert them into this list
 # after/before the appropriate item, rather than replacing the list or
 # assuming absolute positions.
-CATEGORY_ORDER: List[bytes] = [
+CATEGORY_ORDER: list[bytes] = [
     registrar.command.CATEGORY_REPO_CREATION,
     registrar.command.CATEGORY_REMOTE_REPO_MANAGEMENT,
     registrar.command.CATEGORY_COMMITTING,
@@ -95,7 +89,7 @@ CATEGORY_ORDER: List[bytes] = [
 
 # Human-readable category names. These are translated.
 # Extensions with custom categories should add their names here.
-CATEGORY_NAMES: Dict[bytes, bytes] = {
+CATEGORY_NAMES: dict[bytes, bytes] = {
     registrar.command.CATEGORY_REPO_CREATION: b'Repository creation',
     registrar.command.CATEGORY_REMOTE_REPO_MANAGEMENT: b'Remote repository management',
     registrar.command.CATEGORY_COMMITTING: b'Change creation',
@@ -123,7 +117,7 @@ TOPIC_CATEGORY_NONE = b'none'
 # Extensions with custom categories should insert them into this list
 # after/before the appropriate item, rather than replacing the list or
 # assuming absolute positions.
-TOPIC_CATEGORY_ORDER: List[bytes] = [
+TOPIC_CATEGORY_ORDER: list[bytes] = [
     TOPIC_CATEGORY_IDS,
     TOPIC_CATEGORY_OUTPUT,
     TOPIC_CATEGORY_CONFIG,
@@ -133,7 +127,7 @@ TOPIC_CATEGORY_ORDER: List[bytes] = [
 ]
 
 # Human-readable topic category names. These are translated.
-TOPIC_CATEGORY_NAMES: Dict[bytes, bytes] = {
+TOPIC_CATEGORY_NAMES: dict[bytes, bytes] = {
     TOPIC_CATEGORY_IDS: b'Mercurial identifiers',
     TOPIC_CATEGORY_OUTPUT: b'Mercurial output',
     TOPIC_CATEGORY_CONFIG: b'Mercurial configuration',
@@ -145,10 +139,10 @@ TOPIC_CATEGORY_NAMES: Dict[bytes, bytes] = {
 
 def listexts(
     header: bytes,
-    exts: Dict[bytes, bytes],
+    exts: dict[bytes, bytes],
     indent: int = 1,
     showdeprecated: bool = False,
-) -> List[bytes]:
+) -> list[bytes]:
     '''return a text listing of the given extensions'''
     rst = []
     if exts:
@@ -161,6 +155,15 @@ def listexts(
     return rst
 
 
+def ext_help(ui: uimod.ui, ext) -> bytes:
+    doc = pycompat.getdoc(ext)
+    if doc is None:
+        return b""
+    assert doc is not None
+    doc = gettext(doc)
+    return sub_config_item_help(ui, doc)
+
+
 def extshelp(ui: uimod.ui) -> bytes:
     rst = loaddoc(b'extensions')(ui).splitlines(True)
     rst.extend(
@@ -179,7 +182,7 @@ def extshelp(ui: uimod.ui) -> bytes:
     return doc
 
 
-def parsedefaultmarker(text: bytes) -> Optional[Tuple[bytes, List[bytes]]]:
+def parsedefaultmarker(text: bytes) -> tuple[bytes, list[bytes]] | None:
     """given a text 'abc (DEFAULT: def.ghi)',
     returns (b'abc', (b'def', b'ghi')). Otherwise return None"""
     if text[-1:] == b')':
@@ -247,7 +250,7 @@ def optrst(header: bytes, options, verbose: bool, ui: uimod.ui) -> bytes:
 
 
 def indicateomitted(
-    rst: List[bytes], omitted: bytes, notomitted: Optional[bytes] = None
+    rst: list[bytes], omitted: bytes, notomitted: bytes | None = None
 ) -> None:
     rst.append(b'\n\n.. container:: omitted\n\n    %s\n\n' % omitted)
     if notomitted:
@@ -283,7 +286,7 @@ def filtertopic(ui: uimod.ui, topic: bytes) -> bool:
 
 def topicmatch(
     ui: uimod.ui, commands, kw: bytes
-) -> Dict[bytes, List[Tuple[bytes, bytes]]]:
+) -> dict[bytes, list[tuple[bytes, bytes]]]:
     """Return help topics matching kw.
 
     Returns {'section': [(name, summary), ...], ...} where section is
@@ -356,7 +359,7 @@ def topicmatch(
     return results
 
 
-def loaddoc(topic: bytes, subdir: Optional[bytes] = None) -> _DocLoader:
+def loaddoc(topic: bytes, subdir: bytes | None = None) -> _DocLoader:
     """Return a delayed loader for help/topic.txt."""
 
     def loader(ui: uimod.ui) -> bytes:
@@ -367,12 +370,13 @@ def loaddoc(topic: bytes, subdir: Optional[bytes] = None) -> _DocLoader:
             doc = gettext(fp.read())
         for rewriter in helphooks.get(topic, []):
             doc = rewriter(ui, topic, doc)
+        doc = sub_config_item_help(ui, doc)
         return doc
 
     return loader
 
 
-internalstable: List[_HelpEntryNoCategory] = sorted(
+internalstable: list[_HelpEntryNoCategory] = sorted(
     [
         (
             [b'bid-merge'],
@@ -449,7 +453,7 @@ def internalshelp(ui: uimod.ui) -> bytes:
     return b''.join(lines)
 
 
-helptable: List[_HelpEntryWithCategory] = sorted(
+helptable: list[_HelpEntryWithCategory] = sorted(
     [
         (
             [b'bundlespec'],
@@ -611,13 +615,13 @@ helptable: List[_HelpEntryWithCategory] = sorted(
 )
 
 # Maps topics with sub-topics to a list of their sub-topics.
-subtopics: Dict[bytes, List[_HelpEntryNoCategory]] = {
+subtopics: dict[bytes, list[_HelpEntryNoCategory]] = {
     b'internals': internalstable,
 }
 
 # Map topics to lists of callable taking the current topic help and
 # returning the updated version
-helphooks: Dict[bytes, List[_TopicHook]] = {}
+helphooks: dict[bytes, list[_TopicHook]] = {}
 
 
 def addtopichook(topic: bytes, rewriter: _TopicHook) -> None:
@@ -629,7 +633,7 @@ def makeitemsdoc(
     topic: bytes,
     doc: bytes,
     marker: bytes,
-    items: Dict[bytes, bytes],
+    items: dict[bytes, bytes],
     dedent: bool = False,
 ) -> bytes:
     """Extract docstring from the items key to function mapping, build a
@@ -697,9 +701,47 @@ def inserttweakrc(ui: uimod.ui, topic: bytes, doc: bytes) -> bytes:
     return re.sub(br'( *)%s' % re.escape(marker), sub, doc)
 
 
+_CONFIG_DOC_RE = re.compile(b'(^ *)?:config-doc:`([^`]+)`', flags=re.MULTILINE)
+
+
+def sub_config_item_help(ui: uimod.ui, doc: bytes) -> bytes:
+    """replace :config-doc:`foo.bar` markup with the item documentation
+
+    This allow grouping config item declaration and help without having to
+    repeat it in the help text file and keep that in sync.
+    """
+    pieces = []
+    last_match_end = 0
+    for match in _CONFIG_DOC_RE.finditer(doc):
+        # finditer is expected to yield result in order
+        start = match.start()
+        assert last_match_end <= match.start()
+        pieces.append(doc[last_match_end:start])
+        item_name = match.group(2)
+        section, key = item_name.split(b'.', 1)
+        section_items = ui._knownconfig.get(section)
+        if section_items is None:
+            item = None
+        else:
+            item = section_items.get(key)
+        if item is None or not item.documentation:
+            item_doc = b'<missing help text for `%s`>' % item_name
+        else:
+            item_doc = gettext(item.documentation)
+            item_doc = sub_config_item_help(ui, item_doc)
+            indent = match.group(1)
+            if indent:  # either None or 0 should be ignored
+                indent = indent
+                item_doc = indent + item_doc.replace(b'\n', b'\n' + indent)
+        pieces.append(item_doc)
+        last_match_end = match.end()
+    pieces.append(doc[last_match_end:])
+    return b''.join(pieces)
+
+
 def _getcategorizedhelpcmds(
-    ui: uimod.ui, cmdtable, name: bytes, select: Optional[_SelectFn] = None
-) -> Tuple[Dict[bytes, List[bytes]], Dict[bytes, bytes], _SynonymTable]:
+    ui: uimod.ui, cmdtable, name: bytes, select: _SelectFn | None = None
+) -> tuple[dict[bytes, list[bytes]], dict[bytes, bytes], _SynonymTable]:
     # Category -> list of commands
     cats = {}
     # Command -> short description
@@ -729,8 +771,8 @@ def _getcategorizedhelpcmds(
 
 
 def _getcategorizedhelptopics(
-    ui: uimod.ui, topictable: List[_HelpEntry]
-) -> Tuple[Dict[bytes, List[Tuple[bytes, bytes]]], Dict[bytes, List[bytes]]]:
+    ui: uimod.ui, topictable: list[_HelpEntry]
+) -> tuple[dict[bytes, list[tuple[bytes, bytes]]], dict[bytes, list[bytes]]]:
     # Group commands by category.
     topiccats = {}
     syns = {}
@@ -757,9 +799,9 @@ def help_(
     name: bytes,
     unknowncmd: bool = False,
     full: bool = True,
-    subtopic: Optional[bytes] = None,
-    fullname: Optional[bytes] = None,
-    **opts
+    subtopic: bytes | None = None,
+    fullname: bytes | None = None,
+    **opts,
 ) -> bytes:
     """
     Generate the help for 'name' as unformatted restructured text. If
@@ -768,7 +810,7 @@ def help_(
 
     opts = pycompat.byteskwargs(opts)
 
-    def helpcmd(name: bytes, subtopic: Optional[bytes]) -> List[bytes]:
+    def helpcmd(name: bytes, subtopic: bytes | None) -> list[bytes]:
         try:
             aliases, entry = cmdutil.findcmd(
                 name, commands.table, strict=unknowncmd
@@ -810,7 +852,7 @@ def help_(
         doc = gettext(pycompat.getdoc(entry[0]))
         if not doc:
             doc = _(b"(no help text available)")
-        if util.safehasattr(entry[0], b'definition'):  # aliased command
+        if hasattr(entry[0], 'definition'):  # aliased command
             source = entry[0].source
             if entry[0].definition.startswith(b'!'):  # shell alias
                 doc = _(b'shell alias for: %s\n\n%s\n\ndefined by: %s\n') % (
@@ -824,6 +866,7 @@ def help_(
                     doc,
                     source,
                 )
+        doc = sub_config_item_help(ui, doc)
         doc = doc.splitlines(True)
         if ui.quiet or not full:
             rst.append(doc[0])
@@ -869,7 +912,7 @@ def help_(
 
         return rst
 
-    def helplist(select: Optional[_SelectFn] = None, **opts) -> List[bytes]:
+    def helplist(select: _SelectFn | None = None, **opts) -> list[bytes]:
         cats, h, syns = _getcategorizedhelpcmds(
             ui, commands.table, name, select
         )
@@ -994,11 +1037,11 @@ def help_(
                         b"\n(use 'hg help -v%s' to show built-in aliases "
                         b"and global options)\n"
                     )
-                    % (name and b" " + name or b"")
+                    % (b" " + name if name else b"")
                 )
         return rst
 
-    def helptopic(name: bytes, subtopic: Optional[bytes] = None) -> List[bytes]:
+    def helptopic(name: bytes, subtopic: bytes | None = None) -> list[bytes]:
         # Look for sub-topic entry first.
         header, doc = None, None
         if subtopic and name in subtopics:
@@ -1041,15 +1084,18 @@ def help_(
             pass
         return rst
 
-    def helpext(name: bytes, subtopic: Optional[bytes] = None) -> List[bytes]:
+    def helpext(name: bytes, subtopic: bytes | None = None) -> list[bytes]:
         try:
             mod = extensions.find(name)
-            doc = gettext(pycompat.getdoc(mod)) or _(b'no help text available')
+            doc = ext_help(ui, mod)
+            if not doc:
+                doc = _(b'no help text available')
         except KeyError:
             mod = None
             doc = extensions.disabled_help(name)
             if not doc:
                 raise error.UnknownCommand(name)
+            doc = sub_config_item_help(ui, doc)
 
         if b'\n' not in doc:
             head, tail = doc, b""
@@ -1083,9 +1129,7 @@ def help_(
             )
         return rst
 
-    def helpextcmd(
-        name: bytes, subtopic: Optional[bytes] = None
-    ) -> List[bytes]:
+    def helpextcmd(name: bytes, subtopic: bytes | None = None) -> list[bytes]:
         cmd, ext, doc = extensions.disabledcmd(
             ui, name, ui.configbool(b'ui', b'strict')
         )
@@ -1174,11 +1218,11 @@ def help_(
 def formattedhelp(
     ui: uimod.ui,
     commands,
-    fullname: Optional[bytes],
-    keep: Optional[Iterable[bytes]] = None,
+    fullname: bytes | None,
+    keep: Iterable[bytes] | None = None,
     unknowncmd: bool = False,
     full: bool = True,
-    **opts
+    **opts,
 ) -> bytes:
     """get help for a given topic (as a dotted name) as rendered rst
 
@@ -1211,7 +1255,7 @@ def formattedhelp(
         subtopic=subtopic,
         unknowncmd=unknowncmd,
         full=full,
-        **opts
+        **opts,
     )
 
     blocks, pruned = minirst.parse(text, keep=keep)
