@@ -5,6 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
 
 import binascii
 import functools
@@ -12,7 +13,6 @@ import random
 import re
 
 from .i18n import _
-from .pycompat import getattr
 from .node import (
     bin,
     nullrev,
@@ -41,6 +41,7 @@ from . import (
     util,
 )
 from .utils import (
+    dag_util,
     dateutil,
     stringutil,
     urlutil,
@@ -150,6 +151,22 @@ def rawsmartset(repo, subset, x, order):
         return x & subset
 
 
+def raw_node_set(repo, subset, x, order):
+    """argument is a list of nodeid, resolve and use them"""
+    nodes = _ordered_node_set(repo, x)
+    if order == followorder:
+        return subset & nodes
+    else:
+        return nodes & subset
+
+
+def _ordered_node_set(repo, nodes):
+    if not nodes:
+        return baseset()
+    to_rev = repo.changelog.index.rev
+    return baseset([to_rev(r) for r in nodes])
+
+
 def rangeset(repo, subset, x, y, order):
     m = getset(repo, fullreposet(repo), x)
     n = getset(repo, fullreposet(repo), y)
@@ -202,7 +219,7 @@ def _makerangeset(repo, subset, m, n, order):
 
 def dagrange(repo, subset, x, y, order):
     r = fullreposet(repo)
-    xs = dagop.reachableroots(
+    xs = dag_util.reachableroots(
         repo, getset(repo, r, x), getset(repo, r, y), includepath=True
     )
     return subset & xs
@@ -322,12 +339,12 @@ def generationssubrel(repo, subset, x, rel, z, order):
         return baseset()
 
     if ancstart is not None and descstart is not None:
-        s = dagop.revancestors(repo, revs, False, ancstart, ancstop)
-        s += dagop.revdescendants(repo, revs, False, descstart, descstop)
+        s = dag_util.revancestors(repo, revs, False, ancstart, ancstop)
+        s += dag_util.revdescendants(repo, revs, False, descstart, descstop)
     elif ancstart is not None:
-        s = dagop.revancestors(repo, revs, False, ancstart, ancstop)
+        s = dag_util.revancestors(repo, revs, False, ancstart, ancstop)
     elif descstart is not None:
-        s = dagop.revdescendants(repo, revs, False, descstart, descstop)
+        s = dag_util.revdescendants(repo, revs, False, descstart, descstop)
 
     return subset & s
 
@@ -450,7 +467,7 @@ def _ancestors(
     heads = getset(repo, fullreposet(repo), x)
     if not heads:
         return baseset()
-    s = dagop.revancestors(repo, heads, followfirst, startdepth, stopdepth)
+    s = dag_util.revancestors(repo, heads, followfirst, startdepth, stopdepth)
     return subset & s
 
 
@@ -779,7 +796,7 @@ def commonancestors(repo, subset, x):
     if not startrevs:
         return baseset()
     for r in startrevs:
-        subset &= dagop.revancestors(repo, baseset([r]))
+        subset &= dag_util.revancestors(repo, baseset([r]))
     return subset
 
 
@@ -902,7 +919,7 @@ def _descendants(
     roots = getset(repo, fullreposet(repo), x)
     if not roots:
         return baseset()
-    s = dagop.revdescendants(repo, roots, followfirst, startdepth, stopdepth)
+    s = dag_util.revdescendants(repo, roots, followfirst, startdepth, stopdepth)
     return subset & s
 
 
@@ -1238,11 +1255,11 @@ def _follow(repo, subset, x, name, followfirst=False):
                 repo.root, repo.getcwd(), [x], ctx=mctx, default=b'path'
             )
             fctxs.extend(ctx[f].introfilectx() for f in ctx.manifest().walk(m))
-        s = dagop.filerevancestors(fctxs, followfirst)
+        s = dag_util.filerevancestors(fctxs, followfirst)
     else:
         if revs is None:
             revs = baseset([repo[b'.'].rev()])
-        s = dagop.revancestors(repo, revs, followfirst)
+        s = dag_util.revancestors(repo, revs, followfirst)
 
     return subset & s
 
@@ -1319,7 +1336,7 @@ def followlines(repo, subset, x):
         rs = generatorset(
             (
                 c.rev()
-                for c, _linerange in dagop.blockdescendants(
+                for c, _linerange in dag_util.blockdescendants(
                     fctx, fromline, toline
                 )
             ),
@@ -1329,7 +1346,7 @@ def followlines(repo, subset, x):
         rs = generatorset(
             (
                 c.rev()
-                for c, _linerange in dagop.blockancestors(
+                for c, _linerange in dag_util.blockancestors(
                     fctx, fromline, toline
                 )
             ),
@@ -1356,7 +1373,7 @@ def nodefromfile(repo, subset, x):
                 rn = _node(repo, n)
                 if rn is not None:
                     listed_rev.add(rn)
-    except IOError as exc:
+    except OSError as exc:
         m = _(b'cannot open nodes file "%s": %s')
         m %= (path, encoding.strtolocal(exc.strerror))
         raise error.Abort(m)
@@ -1794,7 +1811,7 @@ def only(repo, subset, x):
         if not include:
             return baseset()
 
-        descendants = set(dagop.revdescendants(repo, include, False))
+        descendants = set(dag_util.revdescendants(repo, include, False))
         exclude = [
             rev
             for rev in cl.headrevs()
@@ -1860,7 +1877,7 @@ def outgoing(repo, subset, x):
     l = getargs(x, 0, 1, _(b"outgoing takes one or no arguments"))
     # i18n: "outgoing" is a keyword
     dest = (
-        l and getstring(l[0], _(b"outgoing requires a repository path")) or b''
+        getstring(l[0], _(b"outgoing requires a repository path")) if l else b''
     )
     if dest:
         dests = [dest]
@@ -1967,6 +1984,12 @@ def _phase(repo, subset, *targets):
     return repo._phasecache.getrevset(repo, targets, subset)
 
 
+@predicate(b'_internal()', safe=True)
+def _internal(repo, subset, x):
+    getargs(x, 0, 0, _(b"_internal takes no arguments"))
+    return _phase(repo, subset, *phases.all_internal_phases)
+
+
 @predicate(b'_phase(idx)', safe=True)
 def phase(repo, subset, x):
     l = getargs(x, 1, 1, b"_phase requires one argument")
@@ -2061,7 +2084,7 @@ def present(repo, subset, x, order):
 @predicate(b'_notpublic', safe=True)
 def _notpublic(repo, subset, x):
     getargs(x, 0, 0, b"_notpublic takes no arguments")
-    return _phase(repo, subset, phases.draft, phases.secret)
+    return _phase(repo, subset, *phases.not_public_phases)
 
 
 # for internal use
@@ -2089,7 +2112,7 @@ def _phaseandancestors(repo, subset, x):
     def cutfunc(rev):
         return getphase(repo, rev) < minimalphase
 
-    revs = dagop.revancestors(repo, s, cutfunc=cutfunc)
+    revs = dag_util.revancestors(repo, s, cutfunc=cutfunc)
 
     if phasename == b'draft':  # need to remove secret changesets
         revs = revs.filter(lambda r: getphase(repo, r) == draft)
@@ -2767,6 +2790,7 @@ methods = {
     b"parent": parentspec,
     b"parentpost": parentpost,
     b"smartset": rawsmartset,
+    b"nodeset": raw_node_set,
 }
 
 relations = {
