@@ -1,13 +1,11 @@
+from __future__ import annotations
+
 import array
 import errno
 import fcntl
 import os
 import sys
-
-from typing import (
-    List,
-    Tuple,
-)
+import typing
 
 from . import (
     encoding,
@@ -15,7 +13,7 @@ from . import (
     util,
 )
 
-if pycompat.TYPE_CHECKING:
+if typing.TYPE_CHECKING:
     from . import ui as uimod
 
 # BSD 'more' escapes ANSI color sequences by default. This can be disabled by
@@ -25,7 +23,7 @@ if pycompat.TYPE_CHECKING:
 fallbackpager = b'less'
 
 
-def _rcfiles(path: bytes) -> List[bytes]:
+def _rcfiles(path: bytes) -> list[bytes]:
     rcs = [os.path.join(path, b'hgrc')]
     rcdir = os.path.join(path, b'hgrc.d')
     try:
@@ -41,7 +39,7 @@ def _rcfiles(path: bytes) -> List[bytes]:
     return rcs
 
 
-def systemrcpath() -> List[bytes]:
+def systemrcpath() -> list[bytes]:
     path = []
     if pycompat.sysplatform == b'plan9':
         root = b'lib/mercurial'
@@ -56,11 +54,9 @@ def systemrcpath() -> List[bytes]:
     return path
 
 
-def userrcpath() -> List[bytes]:
+def userrcpath() -> list[bytes]:
     if pycompat.sysplatform == b'plan9':
         return [encoding.environ[b'home'] + b'/lib/hgrc']
-    elif pycompat.isdarwin:
-        return [os.path.expanduser(b'~/.hgrc')]
     else:
         confighome = encoding.environ.get(b'XDG_CONFIG_HOME')
         if confighome is None or not os.path.isabs(confighome):
@@ -72,7 +68,7 @@ def userrcpath() -> List[bytes]:
         ]
 
 
-def termsize(ui: "uimod.ui") -> Tuple[int, int]:
+def termsize(ui: uimod.ui) -> tuple[int, int]:
     try:
         import termios
 
@@ -94,7 +90,7 @@ def termsize(ui: "uimod.ui") -> Tuple[int, int]:
                 return width, height
         except ValueError:
             pass
-        except IOError as e:
+        except OSError as e:
             if e.errno == errno.EINVAL:
                 pass
             else:
