@@ -5,12 +5,16 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+from __future__ import annotations
 
 import struct
-import sys
+import typing
 
-from .. import pycompat
+if typing.TYPE_CHECKING:
+    from typing import (
+        Iterable,
+        Iterator,
+    )
 
 # Very short very of RFC 7049...
 #
@@ -46,23 +50,27 @@ SEMANTIC_TAG_FINITE_SET = 258
 
 # Indefinite types begin with their major type ORd with information value 31.
 BEGIN_INDEFINITE_BYTESTRING = struct.pack(
-    r'>B', MAJOR_TYPE_BYTESTRING << 5 | SUBTYPE_INDEFINITE)
+    '>B', MAJOR_TYPE_BYTESTRING << 5 | SUBTYPE_INDEFINITE
+)
 BEGIN_INDEFINITE_ARRAY = struct.pack(
-    r'>B', MAJOR_TYPE_ARRAY << 5 | SUBTYPE_INDEFINITE)
+    '>B', MAJOR_TYPE_ARRAY << 5 | SUBTYPE_INDEFINITE
+)
 BEGIN_INDEFINITE_MAP = struct.pack(
-    r'>B', MAJOR_TYPE_MAP << 5 | SUBTYPE_INDEFINITE)
+    '>B', MAJOR_TYPE_MAP << 5 | SUBTYPE_INDEFINITE
+)
 
-ENCODED_LENGTH_1 = struct.Struct(r'>B')
-ENCODED_LENGTH_2 = struct.Struct(r'>BB')
-ENCODED_LENGTH_3 = struct.Struct(r'>BH')
-ENCODED_LENGTH_4 = struct.Struct(r'>BL')
-ENCODED_LENGTH_5 = struct.Struct(r'>BQ')
+ENCODED_LENGTH_1 = struct.Struct('>B')
+ENCODED_LENGTH_2 = struct.Struct('>BB')
+ENCODED_LENGTH_3 = struct.Struct('>BH')
+ENCODED_LENGTH_4 = struct.Struct('>BL')
+ENCODED_LENGTH_5 = struct.Struct('>BQ')
 
 # The break ends an indefinite length item.
 BREAK = b'\xff'
 BREAK_INT = 255
 
-def encodelength(majortype, length):
+
+def encodelength(majortype: int, length: int) -> bytes:
     """Obtain a value encoding the major type and its length."""
     if length < 24:
         return ENCODED_LENGTH_1.pack(majortype << 5 | length)
@@ -75,11 +83,13 @@ def encodelength(majortype, length):
     else:
         return ENCODED_LENGTH_5.pack(majortype << 5 | 27, length)
 
-def streamencodebytestring(v):
+
+def streamencodebytestring(v: bytes) -> Iterator[bytes]:
     yield encodelength(MAJOR_TYPE_BYTESTRING, len(v))
     yield v
 
-def streamencodebytestringfromiter(it):
+
+def streamencodebytestringfromiter(it: Iterable[bytes]) -> Iterator[bytes]:
     """Convert an iterator of chunks to an indefinite bytestring.
 
     Given an input that is iterable and each element in the iterator is
@@ -93,7 +103,10 @@ def streamencodebytestringfromiter(it):
 
     yield BREAK
 
-def streamencodeindefinitebytestring(source, chunksize=65536):
+
+def streamencodeindefinitebytestring(
+    source, chunksize: int = 65536
+) -> Iterator[bytes]:
     """Given a large source buffer, emit as an indefinite length bytestring.
 
     This is a generator of chunks constituting the encoded CBOR data.
@@ -104,7 +117,7 @@ def streamencodeindefinitebytestring(source, chunksize=65536):
     l = len(source)
 
     while True:
-        chunk = source[i:i + chunksize]
+        chunk = source[i : i + chunksize]
         i += len(chunk)
 
         yield encodelength(MAJOR_TYPE_BYTESTRING, len(chunk))
@@ -115,7 +128,8 @@ def streamencodeindefinitebytestring(source, chunksize=65536):
 
     yield BREAK
 
-def streamencodeint(v):
+
+def streamencodeint(v: int) -> Iterator[bytes]:
     if v >= 18446744073709551616 or v < -18446744073709551616:
         raise ValueError('big integers not supported')
 
@@ -124,75 +138,75 @@ def streamencodeint(v):
     else:
         yield encodelength(MAJOR_TYPE_NEGINT, abs(v) - 1)
 
-def streamencodearray(l):
+
+def streamencodearray(l) -> Iterator[bytes]:
     """Encode a known size iterable to an array."""
 
     yield encodelength(MAJOR_TYPE_ARRAY, len(l))
 
     for i in l:
-        for chunk in streamencode(i):
-            yield chunk
+        yield from streamencode(i)
 
-def streamencodearrayfromiter(it):
+
+def streamencodearrayfromiter(it) -> Iterator[bytes]:
     """Encode an iterator of items to an indefinite length array."""
 
     yield BEGIN_INDEFINITE_ARRAY
 
     for i in it:
-        for chunk in streamencode(i):
-            yield chunk
+        yield from streamencode(i)
 
     yield BREAK
 
+
 def _mixedtypesortkey(v):
     return type(v).__name__, v
 
-def streamencodeset(s):
+
+def streamencodeset(s) -> Iterator[bytes]:
     # https://www.iana.org/assignments/cbor-tags/cbor-tags.xhtml defines
     # semantic tag 258 for finite sets.
     yield encodelength(MAJOR_TYPE_SEMANTIC, SEMANTIC_TAG_FINITE_SET)
 
-    for chunk in streamencodearray(sorted(s, key=_mixedtypesortkey)):
-        yield chunk
+    yield from streamencodearray(sorted(s, key=_mixedtypesortkey))
 
-def streamencodemap(d):
+
+def streamencodemap(d: dict) -> Iterator[bytes]:
     """Encode dictionary to a generator.
 
     Does not supporting indefinite length dictionaries.
     """
     yield encodelength(MAJOR_TYPE_MAP, len(d))
 
-    for key, value in sorted(d.iteritems(),
-                             key=lambda x: _mixedtypesortkey(x[0])):
-        for chunk in streamencode(key):
-            yield chunk
-        for chunk in streamencode(value):
-            yield chunk
+    for key, value in sorted(d.items(), key=lambda x: _mixedtypesortkey(x[0])):
+        yield from streamencode(key)
+        yield from streamencode(value)
 
-def streamencodemapfromiter(it):
+
+def streamencodemapfromiter(it: Iterable) -> Iterator[bytes]:
     """Given an iterable of (key, value), encode to an indefinite length map."""
     yield BEGIN_INDEFINITE_MAP
 
     for key, value in it:
-        for chunk in streamencode(key):
-            yield chunk
-        for chunk in streamencode(value):
-            yield chunk
+        yield from streamencode(key)
+        yield from streamencode(value)
 
     yield BREAK
 
-def streamencodebool(b):
+
+def streamencodebool(b: bool) -> Iterator[bytes]:
     # major type 7, simple value 20 and 21.
     yield b'\xf5' if b else b'\xf4'
 
-def streamencodenone(v):
+
+def streamencodenone(v: None) -> Iterator[bytes]:
     # major type 7, simple value 22.
     yield b'\xf6'
 
+
 STREAM_ENCODERS = {
     bytes: streamencodebytestring,
     int: streamencodeint,
-    pycompat.long: streamencodeint,
     list: streamencodearray,
     tuple: streamencodearray,
     dict: streamencodemap,
@@ -201,7 +215,8 @@ STREAM_ENCODERS = {
     type(None): streamencodenone,
 }
 
-def streamencode(v):
+
+def streamencode(v) -> Iterator[bytes]:
     """Encode a value in a streaming manner.
 
     Given an input object, encode it to CBOR recursively.
@@ -213,25 +228,32 @@ def streamencode(v):
     """
     fn = STREAM_ENCODERS.get(v.__class__)
 
+    if not fn:
+        # handle subtypes such as encoding.localstr and util.sortdict
+        for ty in STREAM_ENCODERS:
+            if not isinstance(v, ty):
+                continue
+            fn = STREAM_ENCODERS[ty]
+            break
+
     if not fn:
         raise ValueError('do not know how to encode %s' % type(v))
 
     return fn(v)
 
+
 class CBORDecodeError(Exception):
     """Represents an error decoding CBOR."""
 
-if sys.version_info.major >= 3:
-    def _elementtointeger(b, i):
-        return b[i]
-else:
-    def _elementtointeger(b, i):
-        return ord(b[i])
 
-STRUCT_BIG_UBYTE = struct.Struct(r'>B')
-STRUCT_BIG_USHORT = struct.Struct('>H')
-STRUCT_BIG_ULONG = struct.Struct('>L')
-STRUCT_BIG_ULONGLONG = struct.Struct('>Q')
+def _elementtointeger(b, i: int) -> int:
+    return b[i]
+
+
+STRUCT_BIG_UBYTE = struct.Struct('>B')
+STRUCT_BIG_USHORT = struct.Struct(b'>H')
+STRUCT_BIG_ULONG = struct.Struct(b'>L')
+STRUCT_BIG_ULONGLONG = struct.Struct(b'>Q')
 
 SPECIAL_NONE = 0
 SPECIAL_START_INDEFINITE_BYTESTRING = 1
@@ -240,7 +262,8 @@ SPECIAL_START_MAP = 3
 SPECIAL_START_SET = 4
 SPECIAL_INDEFINITE_BREAK = 5
 
-def decodeitem(b, offset=0):
+
+def decodeitem(b, offset: int = 0):
     """Decode a new CBOR value from a buffer at offset.
 
     This function attempts to decode up to one complete CBOR value
@@ -286,6 +309,7 @@ def decodeitem(b, offset=0):
         complete, value, readcount = decodeuint(subtype, b, offset)
 
         if complete:
+            assert value is not None  # help pytype
             return True, -value - 1, readcount + 1, SPECIAL_NONE
         else:
             return False, None, readcount, SPECIAL_NONE
@@ -293,8 +317,9 @@ def decodeitem(b, offset=0):
     elif majortype == MAJOR_TYPE_BYTESTRING:
         # Beginning of bytestrings are treated as uints in order to
         # decode their length, which may be indefinite.
-        complete, size, readcount = decodeuint(subtype, b, offset,
-                                               allowindefinite=True)
+        complete, size, readcount = decodeuint(
+            subtype, b, offset, allowindefinite=True
+        )
 
         # We don't know the size of the bytestring. It must be a definitive
         # length since the indefinite subtype would be encoded in the initial
@@ -306,7 +331,7 @@ def decodeitem(b, offset=0):
         if size is not None:
             # And the data is available in the buffer.
             if offset + readcount + size <= len(b):
-                value = b[offset + readcount:offset + readcount + size]
+                value = b[offset + readcount : offset + readcount + size]
                 return True, value, readcount + size + 1, SPECIAL_NONE
 
             # And we need more data in order to return the bytestring.
@@ -359,15 +384,17 @@ def decodeitem(b, offset=0):
             if offset + readcount >= len(b):
                 return False, None, -1, SPECIAL_NONE
 
-            complete, size, readcount2, special = decodeitem(b,
-                                                             offset + readcount)
+            complete, size, readcount2, special = decodeitem(
+                b, offset + readcount
+            )
 
             if not complete:
                 return False, None, readcount2, SPECIAL_NONE
 
             if special != SPECIAL_START_ARRAY:
-                raise CBORDecodeError('expected array after finite set '
-                                      'semantic tag')
+                raise CBORDecodeError(
+                    'expected array after finite set semantic tag'
+                )
 
             return True, size, readcount + readcount2 + 1, SPECIAL_START_SET
 
@@ -390,7 +417,10 @@ def decodeitem(b, offset=0):
     else:
         assert False
 
-def decodeuint(subtype, b, offset=0, allowindefinite=False):
+
+def decodeuint(
+    subtype: int, b: bytes, offset: int = 0, allowindefinite: bool = False
+):
     """Decode an unsigned integer.
 
     ``subtype`` is the lower 5 bits from the initial byte CBOR item
@@ -420,8 +450,9 @@ def decodeuint(subtype, b, offset=0, allowindefinite=False):
         else:
             raise CBORDecodeError('indefinite length uint not allowed here')
     elif subtype >= 28:
-        raise CBORDecodeError('unsupported subtype on integer type: %d' %
-                              subtype)
+        raise CBORDecodeError(
+            'unsupported subtype on integer type: %d' % subtype
+        )
 
     if subtype == 24:
         s = STRUCT_BIG_UBYTE
@@ -439,6 +470,7 @@ def decodeuint(subtype, b, offset=0, allowindefinite=False):
     else:
         return False, None, len(b) - offset - s.size
 
+
 class bytestringchunk(bytes):
     """Represents a chunk/segment in an indefinite length bytestring.
 
@@ -447,14 +479,18 @@ class bytestringchunk(bytes):
     or last in an indefinite length bytestring.
     """
 
-    def __new__(cls, v, first=False, last=False):
+    isfirst: bool
+    islast: bool
+
+    def __new__(cls, v, first: bool = False, last: bool = False):
         self = bytes.__new__(cls, v)
         self.isfirst = first
         self.islast = last
 
         return self
 
-class sansiodecoder(object):
+
+class sansiodecoder:
     """A CBOR decoder that doesn't perform its own I/O.
 
     To use, construct an instance and feed it segments containing
@@ -519,7 +555,7 @@ class sansiodecoder(object):
     _STATE_WANT_BYTESTRING_CHUNK_FIRST = 5
     _STATE_WANT_BYTESTRING_CHUNK_SUBSEQUENT = 6
 
-    def __init__(self):
+    def __init__(self) -> None:
         # TODO add support for limiting size of bytestrings
         # TODO add support for limiting number of keys / values in collections
         # TODO add support for limiting size of buffered partial values
@@ -539,11 +575,11 @@ class sansiodecoder(object):
         self._decodedvalues = []
 
     @property
-    def inprogress(self):
+    def inprogress(self) -> bool:
         """Whether the decoder has partially decoded a value."""
         return self._state != self._STATE_NONE
 
-    def decode(self, b, offset=0):
+    def decode(self, b, offset: int = 0) -> tuple[bool, int, int]:
         """Attempt to decode bytes from an input buffer.
 
         ``b`` is a collection of bytes and ``offset`` is the byte
@@ -598,40 +634,47 @@ class sansiodecoder(object):
                     self._decodedvalues.append(value)
 
                 elif special == SPECIAL_START_ARRAY:
-                    self._collectionstack.append({
-                        'remaining': value,
-                        'v': [],
-                    })
+                    self._collectionstack.append(
+                        {
+                            b'remaining': value,
+                            b'v': [],
+                        }
+                    )
                     self._state = self._STATE_WANT_ARRAY_VALUE
 
                 elif special == SPECIAL_START_MAP:
-                    self._collectionstack.append({
-                        'remaining': value,
-                        'v': {},
-                    })
+                    self._collectionstack.append(
+                        {
+                            b'remaining': value,
+                            b'v': {},
+                        }
+                    )
                     self._state = self._STATE_WANT_MAP_KEY
 
                 elif special == SPECIAL_START_SET:
-                    self._collectionstack.append({
-                        'remaining': value,
-                        'v': set(),
-                    })
+                    self._collectionstack.append(
+                        {
+                            b'remaining': value,
+                            b'v': set(),
+                        }
+                    )
                     self._state = self._STATE_WANT_SET_VALUE
 
                 elif special == SPECIAL_START_INDEFINITE_BYTESTRING:
                     self._state = self._STATE_WANT_BYTESTRING_CHUNK_FIRST
 
                 else:
-                    raise CBORDecodeError('unhandled special state: %d' %
-                                          special)
+                    raise CBORDecodeError(
+                        'unhandled special state: %d' % special
+                    )
 
             # This value becomes an element of the current array.
             elif self._state == self._STATE_WANT_ARRAY_VALUE:
                 # Simple values get appended.
                 if special == SPECIAL_NONE:
                     c = self._collectionstack[-1]
-                    c['v'].append(value)
-                    c['remaining'] -= 1
+                    c[b'v'].append(value)
+                    c[b'remaining'] -= 1
 
                     # self._state doesn't need changed.
 
@@ -640,13 +683,15 @@ class sansiodecoder(object):
                     lastc = self._collectionstack[-1]
                     newvalue = []
 
-                    lastc['v'].append(newvalue)
-                    lastc['remaining'] -= 1
+                    lastc[b'v'].append(newvalue)
+                    lastc[b'remaining'] -= 1
 
-                    self._collectionstack.append({
-                        'remaining': value,
-                        'v': newvalue,
-                    })
+                    self._collectionstack.append(
+                        {
+                            b'remaining': value,
+                            b'v': newvalue,
+                        }
+                    )
 
                     # self._state doesn't need changed.
 
@@ -655,13 +700,12 @@ class sansiodecoder(object):
                     lastc = self._collectionstack[-1]
                     newvalue = {}
 
-                    lastc['v'].append(newvalue)
-                    lastc['remaining'] -= 1
+                    lastc[b'v'].append(newvalue)
+                    lastc[b'remaining'] -= 1
 
-                    self._collectionstack.append({
-                        'remaining': value,
-                        'v': newvalue
-                    })
+                    self._collectionstack.append(
+                        {b'remaining': value, b'v': newvalue}
+                    )
 
                     self._state = self._STATE_WANT_MAP_KEY
 
@@ -669,23 +713,29 @@ class sansiodecoder(object):
                     lastc = self._collectionstack[-1]
                     newvalue = set()
 
-                    lastc['v'].append(newvalue)
-                    lastc['remaining'] -= 1
+                    lastc[b'v'].append(newvalue)
+                    lastc[b'remaining'] -= 1
 
-                    self._collectionstack.append({
-                        'remaining': value,
-                        'v': newvalue,
-                    })
+                    self._collectionstack.append(
+                        {
+                            b'remaining': value,
+                            b'v': newvalue,
+                        }
+                    )
 
                     self._state = self._STATE_WANT_SET_VALUE
 
                 elif special == SPECIAL_START_INDEFINITE_BYTESTRING:
-                    raise CBORDecodeError('indefinite length bytestrings '
-                                          'not allowed as array values')
+                    raise CBORDecodeError(
+                        'indefinite length bytestrings '
+                        'not allowed as array values'
+                    )
 
                 else:
-                    raise CBORDecodeError('unhandled special item when '
-                                          'expecting array value: %d' % special)
+                    raise CBORDecodeError(
+                        'unhandled special item when '
+                        'expecting array value: %d' % special
+                    )
 
             # This value becomes the key of the current map instance.
             elif self._state == self._STATE_WANT_MAP_KEY:
@@ -694,26 +744,34 @@ class sansiodecoder(object):
                     self._state = self._STATE_WANT_MAP_VALUE
 
                 elif special == SPECIAL_START_INDEFINITE_BYTESTRING:
-                    raise CBORDecodeError('indefinite length bytestrings '
-                                          'not allowed as map keys')
+                    raise CBORDecodeError(
+                        'indefinite length bytestrings '
+                        'not allowed as map keys'
+                    )
 
-                elif special in (SPECIAL_START_ARRAY, SPECIAL_START_MAP,
-                                 SPECIAL_START_SET):
-                    raise CBORDecodeError('collections not supported as map '
-                                          'keys')
+                elif special in (
+                    SPECIAL_START_ARRAY,
+                    SPECIAL_START_MAP,
+                    SPECIAL_START_SET,
+                ):
+                    raise CBORDecodeError(
+                        'collections not supported as map keys'
+                    )
 
                 # We do not allow special values to be used as map keys.
                 else:
-                    raise CBORDecodeError('unhandled special item when '
-                                          'expecting map key: %d' % special)
+                    raise CBORDecodeError(
+                        'unhandled special item when '
+                        'expecting map key: %d' % special
+                    )
 
             # This value becomes the value of the current map key.
             elif self._state == self._STATE_WANT_MAP_VALUE:
                 # Simple values simply get inserted into the map.
                 if special == SPECIAL_NONE:
                     lastc = self._collectionstack[-1]
-                    lastc['v'][self._currentmapkey] = value
-                    lastc['remaining'] -= 1
+                    lastc[b'v'][self._currentmapkey] = value
+                    lastc[b'remaining'] -= 1
 
                     self._state = self._STATE_WANT_MAP_KEY
 
@@ -722,13 +780,15 @@ class sansiodecoder(object):
                     lastc = self._collectionstack[-1]
                     newvalue = []
 
-                    lastc['v'][self._currentmapkey] = newvalue
-                    lastc['remaining'] -= 1
+                    lastc[b'v'][self._currentmapkey] = newvalue
+                    lastc[b'remaining'] -= 1
 
-                    self._collectionstack.append({
-                        'remaining': value,
-                        'v': newvalue,
-                    })
+                    self._collectionstack.append(
+                        {
+                            b'remaining': value,
+                            b'v': newvalue,
+                        }
+                    )
 
                     self._state = self._STATE_WANT_ARRAY_VALUE
 
@@ -737,13 +797,15 @@ class sansiodecoder(object):
                     lastc = self._collectionstack[-1]
                     newvalue = {}
 
-                    lastc['v'][self._currentmapkey] = newvalue
-                    lastc['remaining'] -= 1
+                    lastc[b'v'][self._currentmapkey] = newvalue
+                    lastc[b'remaining'] -= 1
 
-                    self._collectionstack.append({
-                        'remaining': value,
-                        'v': newvalue,
-                    })
+                    self._collectionstack.append(
+                        {
+                            b'remaining': value,
+                            b'v': newvalue,
+                        }
+                    )
 
                     self._state = self._STATE_WANT_MAP_KEY
 
@@ -752,23 +814,29 @@ class sansiodecoder(object):
                     lastc = self._collectionstack[-1]
                     newvalue = set()
 
-                    lastc['v'][self._currentmapkey] = newvalue
-                    lastc['remaining'] -= 1
+                    lastc[b'v'][self._currentmapkey] = newvalue
+                    lastc[b'remaining'] -= 1
 
-                    self._collectionstack.append({
-                        'remaining': value,
-                        'v': newvalue,
-                    })
+                    self._collectionstack.append(
+                        {
+                            b'remaining': value,
+                            b'v': newvalue,
+                        }
+                    )
 
                     self._state = self._STATE_WANT_SET_VALUE
 
                 elif special == SPECIAL_START_INDEFINITE_BYTESTRING:
-                    raise CBORDecodeError('indefinite length bytestrings not '
-                                          'allowed as map values')
+                    raise CBORDecodeError(
+                        'indefinite length bytestrings not '
+                        'allowed as map values'
+                    )
 
                 else:
-                    raise CBORDecodeError('unhandled special item when '
-                                          'expecting map value: %d' % special)
+                    raise CBORDecodeError(
+                        'unhandled special item when '
+                        'expecting map value: %d' % special
+                    )
 
                 self._currentmapkey = None
 
@@ -776,31 +844,39 @@ class sansiodecoder(object):
             elif self._state == self._STATE_WANT_SET_VALUE:
                 if special == SPECIAL_NONE:
                     lastc = self._collectionstack[-1]
-                    lastc['v'].add(value)
-                    lastc['remaining'] -= 1
+                    lastc[b'v'].add(value)
+                    lastc[b'remaining'] -= 1
 
                 elif special == SPECIAL_START_INDEFINITE_BYTESTRING:
-                    raise CBORDecodeError('indefinite length bytestrings not '
-                                          'allowed as set values')
+                    raise CBORDecodeError(
+                        'indefinite length bytestrings not '
+                        'allowed as set values'
+                    )
 
-                elif special in (SPECIAL_START_ARRAY,
-                                 SPECIAL_START_MAP,
-                                 SPECIAL_START_SET):
-                    raise CBORDecodeError('collections not allowed as set '
-                                          'values')
+                elif special in (
+                    SPECIAL_START_ARRAY,
+                    SPECIAL_START_MAP,
+                    SPECIAL_START_SET,
+                ):
+                    raise CBORDecodeError(
+                        'collections not allowed as set values'
+                    )
 
                 # We don't allow non-trivial types to exist as set values.
                 else:
-                    raise CBORDecodeError('unhandled special item when '
-                                          'expecting set value: %d' % special)
+                    raise CBORDecodeError(
+                        'unhandled special item when '
+                        'expecting set value: %d' % special
+                    )
 
             # This value represents the first chunk in an indefinite length
             # bytestring.
             elif self._state == self._STATE_WANT_BYTESTRING_CHUNK_FIRST:
                 # We received a full chunk.
                 if special == SPECIAL_NONE:
-                    self._decodedvalues.append(bytestringchunk(value,
-                                                               first=True))
+                    self._decodedvalues.append(
+                        bytestringchunk(value, first=True)
+                    )
 
                     self._state = self._STATE_WANT_BYTESTRING_CHUNK_SUBSEQUENT
 
@@ -810,9 +886,9 @@ class sansiodecoder(object):
                     # We /could/ convert this to a b''. But we want to preserve
                     # the nature of the underlying data so consumers expecting
                     # an indefinite length bytestring get one.
-                    self._decodedvalues.append(bytestringchunk(b'',
-                                                               first=True,
-                                                               last=True))
+                    self._decodedvalues.append(
+                        bytestringchunk(b'', first=True, last=True)
+                    )
 
                     # Since indefinite length bytestrings can't be used in
                     # collections, we must be at the root level.
@@ -820,9 +896,10 @@ class sansiodecoder(object):
                     self._state = self._STATE_NONE
 
                 else:
-                    raise CBORDecodeError('unexpected special value when '
-                                          'expecting bytestring chunk: %d' %
-                                          special)
+                    raise CBORDecodeError(
+                        'unexpected special value when '
+                        'expecting bytestring chunk: %d' % special
+                    )
 
             # This value represents the non-initial chunk in an indefinite
             # length bytestring.
@@ -841,27 +918,31 @@ class sansiodecoder(object):
                     self._state = self._STATE_NONE
 
                 else:
-                    raise CBORDecodeError('unexpected special value when '
-                                          'expecting bytestring chunk: %d' %
-                                          special)
+                    raise CBORDecodeError(
+                        'unexpected special value when '
+                        'expecting bytestring chunk: %d' % special
+                    )
 
             else:
-                raise CBORDecodeError('unhandled decoder state: %d' %
-                                      self._state)
+                raise CBORDecodeError(
+                    'unhandled decoder state: %d' % self._state
+                )
 
             # We could have just added the final value in a collection. End
             # all complete collections at the top of the stack.
             while True:
                 # Bail if we're not waiting on a new collection item.
-                if self._state not in (self._STATE_WANT_ARRAY_VALUE,
-                                       self._STATE_WANT_MAP_KEY,
-                                       self._STATE_WANT_SET_VALUE):
+                if self._state not in (
+                    self._STATE_WANT_ARRAY_VALUE,
+                    self._STATE_WANT_MAP_KEY,
+                    self._STATE_WANT_SET_VALUE,
+                ):
                     break
 
                 # Or we are expecting more items for this collection.
                 lastc = self._collectionstack[-1]
 
-                if lastc['remaining']:
+                if lastc[b'remaining']:
                     break
 
                 # The collection at the top of the stack is complete.
@@ -878,11 +959,11 @@ class sansiodecoder(object):
                         list: self._STATE_WANT_ARRAY_VALUE,
                         dict: self._STATE_WANT_MAP_KEY,
                         set: self._STATE_WANT_SET_VALUE,
-                    }[type(self._collectionstack[-1]['v'])]
+                    }[type(self._collectionstack[-1][b'v'])]
 
                 # If this is the root collection, emit it.
                 else:
-                    self._decodedvalues.append(lastc['v'])
+                    self._decodedvalues.append(lastc[b'v'])
                     self._state = self._STATE_NONE
 
         return (
@@ -901,7 +982,8 @@ class sansiodecoder(object):
         self._decodedvalues = []
         return l
 
-class bufferingdecoder(object):
+
+class bufferingdecoder:
     """A CBOR decoder that buffers undecoded input.
 
     This is a glorified wrapper around ``sansiodecoder`` that adds a buffering
@@ -911,12 +993,17 @@ class bufferingdecoder(object):
     TODO consider adding limits as to the maximum amount of data that can
     be buffered.
     """
-    def __init__(self):
+
+    _decoder: sansiodecoder
+    _chunks: list
+    _wanted: int
+
+    def __init__(self) -> None:
         self._decoder = sansiodecoder()
         self._chunks = []
         self._wanted = 0
 
-    def decode(self, b):
+    def decode(self, b) -> tuple[bool, int, int]:
         """Attempt to decode bytes to CBOR values.
 
         Returns a tuple with the following fields:
@@ -970,6 +1057,7 @@ class bufferingdecoder(object):
     def getavailable(self):
         return self._decoder.getavailable()
 
+
 def decodeall(b):
     """Decode all CBOR items present in an iterable of bytes.
 
