@@ -13,10 +13,6 @@ import socket
 
 from typing import (
     Callable,
-    Dict,
-    Optional,
-    Tuple,
-    Union,
 )
 
 from ..i18n import _
@@ -40,12 +36,14 @@ from ..revlogutils import (
 )
 
 # keeps pyflakes happy
-assert [Callable, Dict, Tuple, Union]
+assert [
+    Callable,
+]
 
 urlreq = urllibcompat.urlreq
 
 
-def getport(port: Union[bytes, int]) -> int:
+def getport(port: bytes | int) -> int:
     """Return the port for a given network service.
 
     If port is an integer, it's returned as is. If it's a string, it's
@@ -663,7 +661,7 @@ class paths(dict):
             self[name] = new_paths
 
 
-_pathsuboptions: Dict[bytes, Tuple[str, Callable]] = {}
+_pathsuboptions: dict[bytes, tuple[str, Callable]] = {}
 # a dictionnary of methods that can be used to format a sub-option value
 path_suboptions_display = {}
 
@@ -893,7 +891,7 @@ class path(int_misc.IPath):
         self.rawloc = rawloc
         self.loc = b'%s' % u
 
-    def copy(self, new_raw_location: Optional[bytes] = None) -> path:
+    def copy(self, new_raw_location: bytes | None = None) -> path:
         """make a copy of this path object
 
         When `new_raw_location` is set, the new path will point to it.
@@ -966,7 +964,7 @@ class path(int_misc.IPath):
             return False
 
     @property
-    def suboptions(self) -> Dict[bytes, bytes]:
+    def suboptions(self) -> dict[bytes, bytes]:
         """Return sub-options and their values for this path.
 
         This is intended to be used for presentation purposes.
