@@ -3,28 +3,28 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+from __future__ import annotations
+
+import abc
+
+from typing import (
+    Protocol,
+)
 
 from .node import (
     bin,
     hex,
 )
 from .i18n import _
-from .pycompat import getattr
-from .thirdparty import attr
+
 from . import (
     error,
     util,
 )
-from .interfaces import util as interfaceutil
 from .utils import compression
 
 # Names of the SSH protocol implementations.
 SSHV1 = b'ssh-v1'
-# These are advertised over the wire. Increment the counters at the end
-# to reflect BC breakages.
-SSHV2 = b'exp-ssh-v2-0003'
-HTTP_WIREPROTO_V2 = b'exp-http-v2-0003'
 
 NARROWCAP = b'exp-narrow-1'
 ELLIPSESCAP1 = b'exp-ellipses-1'
@@ -37,30 +37,21 @@ TRANSPORTS = {
         b'transport': b'ssh',
         b'version': 1,
     },
-    SSHV2: {
-        b'transport': b'ssh',
-        # TODO mark as version 2 once all commands are implemented.
-        b'version': 1,
-    },
     b'http-v1': {
         b'transport': b'http',
         b'version': 1,
     },
-    HTTP_WIREPROTO_V2: {
-        b'transport': b'http',
-        b'version': 2,
-    },
 }
 
 
-class bytesresponse(object):
+class bytesresponse:
     """A wire protocol response consisting of raw bytes."""
 
     def __init__(self, data):
         self.data = data
 
 
-class ooberror(object):
+class ooberror:
     """wireproto reply: failure of a batch of operation
 
     Something failed during a batch call. The error message is stored in
@@ -71,7 +62,7 @@ class ooberror(object):
         self.message = message
 
 
-class pushres(object):
+class pushres:
     """wireproto reply: success with simple integer return
 
     The call was successful and returned an integer contained in `self.res`.
@@ -82,7 +73,7 @@ class pushres(object):
         self.output = output
 
 
-class pusherr(object):
+class pusherr:
     """wireproto reply: failure
 
     The call failed. The `self.res` attribute contains the error message.
@@ -93,7 +84,7 @@ class pusherr(object):
         self.output = output
 
 
-class streamres(object):
+class streamres:
     """wireproto reply: binary stream
 
     The call was successful and the result is a stream.
@@ -110,7 +101,7 @@ class streamres(object):
         self.prefer_uncompressed = prefer_uncompressed
 
 
-class streamreslegacy(object):
+class streamreslegacy:
     """wireproto reply: uncompressed binary stream
 
     The call was successful and the result is a stream.
@@ -187,7 +178,7 @@ GETBUNDLE_ARGUMENTS = {
 }
 
 
-class baseprotocolhandler(interfaceutil.Interface):
+class baseprotocolhandler(Protocol):
     """Abstract base class for wire protocol handlers.
 
     A wire protocol handler serves as an interface between protocol command
@@ -196,14 +187,16 @@ class baseprotocolhandler(interfaceutil.Interface):
     the request, handle response types, etc.
     """
 
-    name = interfaceutil.Attribute(
+    @property
+    @abc.abstractmethod
+    def name(self) -> bytes:
         """The name of the protocol implementation.
 
         Used for uniquely identifying the transport type.
         """
-    )
 
-    def getargs(args):
+    @abc.abstractmethod
+    def getargs(self, args):
         """return the value for arguments in <args>
 
         For version 1 transports, returns a list of values in the same
@@ -211,20 +204,23 @@ class baseprotocolhandler(interfaceutil.Interface):
         a dict mapping argument name to value.
         """
 
-    def getprotocaps():
+    @abc.abstractmethod
+    def getprotocaps(self):
         """Returns the list of protocol-level capabilities of client
 
         Returns a list of capabilities as declared by the client for
         the current request (or connection for stateful protocol handlers)."""
 
-    def getpayload():
+    @abc.abstractmethod
+    def getpayload(self):
         """Provide a generator for the raw payload.
 
         The caller is responsible for ensuring that the full payload is
         processed.
         """
 
-    def mayberedirectstdio():
+    @abc.abstractmethod
+    def mayberedirectstdio(self):
         """Context manager to possibly redirect stdio.
 
         The context manager yields a file-object like object that receives
@@ -237,10 +233,12 @@ class baseprotocolhandler(interfaceutil.Interface):
         won't be captured.
         """
 
-    def client():
+    @abc.abstractmethod
+    def client(self):
         """Returns a string representation of this client (as bytes)."""
 
-    def addcapabilities(repo, caps):
+    @abc.abstractmethod
+    def addcapabilities(self, repo, caps):
         """Adds advertised capabilities specific to this protocol.
 
         Receives the list of capabilities collected so far.
@@ -248,7 +246,8 @@ class baseprotocolhandler(interfaceutil.Interface):
         Returns a list of capabilities. The passed in argument can be returned.
         """
 
-    def checkperm(perm):
+    @abc.abstractmethod
+    def checkperm(self, perm):
         """Validate that the client has permissions to perform a request.
 
         The argument is the permission required to proceed. If the client
@@ -257,7 +256,7 @@ class baseprotocolhandler(interfaceutil.Interface):
         """
 
 
-class commandentry(object):
+class commandentry:
     """Represents a declared wire protocol command."""
 
     def __init__(
@@ -340,7 +339,7 @@ class commanddict(dict):
                 b'or 2-tuples'
             )
 
-        return super(commanddict, self).__setitem__(k, v)
+        return super().__setitem__(k, v)
 
     def commandavailable(self, command, proto):
         """Determine if a command is available for the requested protocol."""
@@ -381,9 +380,7 @@ def supportedcompengines(ui, role):
     # No explicit config. Filter out the ones that aren't supposed to be
     # advertised and return default ordering.
     if not configengines:
-        attr = (
-            b'serverpriority' if role == util.SERVERROLE else b'clientpriority'
-        )
+        attr = 'serverpriority' if role == util.SERVERROLE else 'clientpriority'
         return [
             e for e in compengines if getattr(e.wireprotosupport(), attr) > 0
         ]
@@ -413,51 +410,7 @@ def supportedcompengines(ui, role):
             )
             % config,
             hint=_(b'usable compression engines: %s')
-            % b', '.sorted(validnames),  # pytype: disable=attribute-error
+            % b', '.join(sorted(validnames)),
         )
 
     return compengines
-
-
-@attr.s
-class encodedresponse(object):
-    """Represents response data that is already content encoded.
-
-    Wire protocol version 2 only.
-
-    Commands typically emit Python objects that are encoded and sent over the
-    wire. If commands emit an object of this type, the encoding step is bypassed
-    and the content from this object is used instead.
-    """
-
-    data = attr.ib()
-
-
-@attr.s
-class alternatelocationresponse(object):
-    """Represents a response available at an alternate location.
-
-    Instances are sent in place of actual response objects when the server
-    is sending a "content redirect" response.
-
-    Only compatible with wire protocol version 2.
-    """
-
-    url = attr.ib()
-    mediatype = attr.ib()
-    size = attr.ib(default=None)
-    fullhashes = attr.ib(default=None)
-    fullhashseed = attr.ib(default=None)
-    serverdercerts = attr.ib(default=None)
-    servercadercerts = attr.ib(default=None)
-
-
-@attr.s
-class indefinitebytestringresponse(object):
-    """Represents an object to be encoded to an indefinite length bytestring.
-
-    Instances are initialized from an iterable of chunks, with each chunk being
-    a bytes instance.
-    """
-
-    chunks = attr.ib()
